/* Query.c
   Ein Teil von FoxIRC (C)by Dario Abatianni (eisfuchs@tigress.com)

   Funktionen fr die Query-Fenster */

#include <forms.h>

#include "Channel.h"
#include "Constants.h"
#include "Query.h"
#include "Preferences.h"
#include "ErrorLog.h"

extern struct preferences prefs;
extern struct server *server;
extern int maxhistory;

int queryfilter(FL_OBJECT *input, int event,FL_Coord mx, FL_Coord my,int key, void *raw_event)
{
  char outstring[LINELEN+NICKLEN+5];
  int count;
  struct query *query;

  if(event==FL_KEYBOARD)
  {
    query=input->u_vdata;

    switch(key)
    {
      case 13:
        strcpy(outstring,fl_get_input(input));

        if(strlen(outstring)==0) return FL_PREEMPT;
        else
        {
          resetautoawaytimer(&prefs);
          if(prefs.isaway)
          {
            transmitraw(server->sock,"AWAY");
            prefs.isaway=0;
          }

          server->histline=maxhistory-1;
          strcpy(server->history[server->histline],outstring);
          for(count=1;count!=maxhistory;count++) strcpy(server->history[count-1],server->history[count]);

          server->history[server->histline][0]=0;

          sprintf(outstring,"/MSG %s %s",query->name,fl_get_input(input));
          transmit(server,outstring);

          fl_set_input(input,"");

          return 0;
        }
      break;

      case 65362:   // CRSR_UP
        if(server!=0)
        {
          if(server->histline!=0) fl_set_input(input,server->history[--server->histline]);
          return FL_PREEMPT;
        }
      break;

      case 65364:   // CRSR_DN
        if(server!=0)
        {
          if(server->histline!=(maxhistory-1)) fl_set_input(input,server->history[++server->histline]);
          return FL_PREEMPT;
        }
      break;

      case 65365:   // PG_UP
      {
        int height,topline,newtopline;
        FL_OBJECT *browser;

        browser=query->window->QueryText;

        height=fl_get_browser_screenlines(browser);
        topline=fl_get_browser_topline(browser);

        newtopline=topline-height+1;
        if(newtopline<1) newtopline=1;
        
        fl_set_browser_topline(browser,newtopline);
      } 
      break;

      case 65366:   // PG_DN
      {
        int height,topline,newtopline,maxline;
        FL_OBJECT *browser;

        browser=query->window->QueryText;

        height=fl_get_browser_screenlines(browser);
        topline=fl_get_browser_topline(browser);
        maxline=fl_get_browser_maxline(browser);

        newtopline=topline+height-1;
        if(newtopline>(maxline-height+1)) newtopline=maxline-height+1;
        
        fl_set_browser_topline(browser,newtopline);
      }
      break;
/*
  War wohl keine so gute Idee ;))

      case 65360:   // POS_1
      {
        FL_OBJECT *browser;

        browser=query->window->QueryText;
        fl_set_browser_topline(browser,1);
      }
      break;

      case 65367:   // ENDE
      {
        FL_OBJECT *browser;
        int height,topline,newtopline,maxline;

        browser=query->window->QueryText;

        height=fl_get_browser_screenlines(browser);
        maxline=fl_get_browser_maxline(browser);
        fl_set_browser_topline(browser,maxline-height+1);
      }
      break;
*/
    } // endswitch
  }
  return 0;
}

struct node *addquery(struct server *server,char *nick)
{
  struct node *newquerynode;
  struct query *newquery;
  char hostmask[NICKLEN+128];

  newquerynode=0;

  if(isignore(prefs.ignorelist,nick,1)==0)
  {
    if(server->querylist==0) server->querylist=newlist();
    if(server->querylist==0) logerror(errorlogwindow,"addquery(): server->querylist==0!\n");
    else
    {
      newquerynode=addnewnodetolist(server->querylist,sizeof(struct query));

      if(newquerynode==0) logerror(errorlogwindow,"addquery(): newquerynode==0!\n");
      else
      {
        newquery=newquerynode->data;

        strcpy(newquery->name,nick);
        strcpy(newquery->logname,nick);                     // falls sich der Nick whrend des Querys ndert
        sprintf(hostmask,"%s (noch nicht bekannt)",nick);
        newquery->logstart=1;

        newquery->window=create_form_QueryWindow();
        newquery->querytext=newquery->window->QueryText;

        fl_set_object_label(newquery->window->Hostmask,hostmask);
        fl_set_button(newquery->window->Log,prefs.log);
        fl_set_browser_specialkey(newquery->querytext,'\\');

        fl_set_form_atclose(newquery->window->QueryWindow,closequery,newquerynode);

        fl_set_form_icon(newquery->window->QueryWindow,iconid,maskid);
        fl_show_form(newquery->window->QueryWindow,FL_PLACE_POSITION,FL_FULLBORDER,nick);

        newquery->server=server;

        newquery->window->Input->wantkey=FL_KEY_TAB;
        newquery->window->Input->u_vdata=newquery;
        fl_set_object_prehandler(newquery->window->Input,queryfilter);
      }
    }
  }

  return newquerynode;
}

struct node *isquery(struct server *server,char *name)
{
  struct node *querynode;
  struct query *query;
  int quit;

  querynode=0;

  if(server->querylist)
  {
    querynode=server->querylist->head;
    if(querynode)
    {
      quit=0;

      while(querynode!=0 && quit==0)
      {
        query=querynode->data;
        if(strcasecmp(name,query->name)==0) quit=1;
        if(quit==0) querynode=querynode->next;
      } // endwhile
    }
  }

  return querynode;
}

struct node *getquerybyname(struct server *server,char *name)
{
  struct node *querynode;

  querynode=isquery(server,name);

  if(querynode==0)
  {
    if(prefs.openquery!=0) querynode=addquery(server,name);
  }

  return querynode;
}

void removequery(struct node *querynode)
{
  struct query *query;

  if(querynode==0) logerror(errorlogwindow,"removequery(): query==0!");
  else
  {
    query=querynode->data;
    fl_hide_form(query->window->QueryWindow);

    freenode(querynode);
  }
}

void removeallqueries(struct server *server)
{
  struct node *querynode;

  if(server->querylist)
  {
    querynode=server->querylist->head;

    while(querynode)
    {
      querynode=querynode->next;
      removequery(server->querylist->head);
    } // endwhile
  }
}

int closequery(FL_FORM *form,void *querynode)
{
  removequery(querynode);
  return FL_IGNORE;
}

void handlequeries(struct server *server,FL_OBJECT *select)
{
  struct node *querynode;
  struct query *query;
  char whois[LINELEN];

  if(server->querylist)
  {
    querynode=server->querylist->head;

    while(querynode && select)
    {
      query=querynode->data;
      if(select==query->window->Whois)
      {
        sprintf(whois,"/WHOIS %s",query->name);
        transmit(server,whois);
      }

      if(select!=0) querynode=querynode->next;
    } // endwhile
  }
}
