/* Utils.c
   Ein Teil von FoxIRC (C)by Dario Abatianni (eisfuchs@tigress.com)

   Hilfsfunktionen */

#include <X11/Xlib.h>

#include "Utils.h"
#include "Constants.h"
#include "Preferences.h"
#include "ErrorLog.h"

#include "../Forms/MainWindow.h"

extern FD_MainWindow *mainform;

void deactivate(FL_OBJECT *obj)
{
  fl_deactivate_object(obj);
  fl_set_object_lcol(obj,FL_MCOL);
}

void activate(FL_OBJECT *obj)
{
  fl_set_object_lcol(obj,FL_BLACK);
  fl_activate_object(obj);
}

void lowerstr(char *string)
{
  int count;

  for(count=0;count<strlen(string);count++) string[count]=tolower(string[count]);
}

int geometryevent(FL_FORM *form,void *raw_event)
{
  XConfigureEvent *event;
  int rc;

  event=raw_event;

//  printf("%d %d %d\n",event->type,ConfigureNotify,Expose);

  if(event->type==ConfigureNotify)
  {
    struct geometry *geo;

    geo=0;

    if(form->u_ldata==WINDOW_MAIN) geo=&prefs.geomain;
    if(form->u_ldata==WINDOW_ISON) geo=&prefs.geoison;
    if(geo)
    {
      /* This code was suggested by Jens Thoms Toerring <Jens.Toerring@physik.fu-berlin.de> */
      /* Thanks! */
      int temp;

      XWindowAttributes attr;
      Window win, wdummy1, *wdummy2;

      win = form->window;
//      XQueryTree( fl_get_display(), win, &wdummy1, &win, &wdummy2, &temp );
      XQueryTree( fl_get_display(), win, &wdummy1, &win, &wdummy2, &temp );
      XGetWindowAttributes( fl_get_display(), win, &attr );

      geo->x=attr.x;
      geo->y=attr.y;

//      geo->x=event->x;   /* Das hier gibt mir die Werte ohne den Window-Manager */
//      geo->y=event->y;   /* Ist also nicht brauchbar */

      geo->w=event->width;
      geo->h=event->height;

//      printf("%d %d %d %d\n",geo->x,geo->y,geo->w,geo->h);
    }
  }

  return 0;
}

int lineexists(FL_OBJECT *browser,const char *string,int casesensitive)
{
  int found,count,max;

  found=0;
  max=fl_get_browser_maxline(browser);

  for(count=1;count<=max;count++)
  {
    if(strcmp(fl_get_browser_line(browser,count),string)==0) found=count;
  }

  return found;
}

int sort2(const void *a,const void *b)   /* Um eine Warning bei sortbrowser() zu vehindern */
{
  return strcasecmp(a,b);
}

void catch(char *string)
{
  char searchbuffer[LINELEN],*needle[7]={"http://","https://","ftp://","gopher://","www.","ftp.",0};
  int needlecount;

  /* erzeuge suchstring in Kleinbuchstaben */
  strcpy(searchbuffer,string);
  lowerstr(searchbuffer);

  /* Protokollsuche zurcksetzen */
  needlecount=0;
  while(needle[needlecount]!=0)
  {
    char *search,*url;
    int offset;

    /* Originalstring-Offset auf 0 setzen */
    offset=0;
    /* Pointer, der verbogen werden darf, initialisieren */
    search=searchbuffer;

    do
    {
      // printf("%s\n",search);
      /* Nach Protokollnamen suchen */
      url=strstr(search,needle[needlecount]);
      if(url!=0)
      {
        int len,pos;
        /* Position im Suchstring finden */
        pos=url-search;
        len=0;
        /* Lnge der URL ermitteln */
        while(url[len]>32) len++;
        len--;
        /* Unerwnschte Zeichen am Ende entfernen */
        if(prefs.smartcatch==1)
        {
          while(url[len]=='.' ||
                url[len]==',' ||
                url[len]=='(' ||
                url[len]==')' ||
                url[len]=='\'' ||
                url[len]=='\"') len--;
        }

        /* URL hinzufgen, aus dem Originalstring + offset + Suchposition pos */
        addurl(string+pos+offset,len+1);
        /* Gefundene URL lschen um doppelte Suchtreffer zu vermeiden */
        memset(url,32,len+1);
        /* Suchstring hinter die URL setzen */
        search=url+len;
        /* Originalstring-Offset anpassen */
        offset+=pos+len;
      }
    } while(url);    /* Solange ein Protokoll gefunden wurde, weitersuchen */
    needlecount++;   /* nchstes Protokoll vorbereiten und suchen */
  }
}

unsigned long intel(unsigned long value)
{
  value=((value & 0xff000000) >> 24) +
        ((value & 0xff0000) >> 8) +
        ((value & 0xff00) << 8) +
        ((value & 0xff) << 24);

  return value;
}

void swapnodes(void *node1,void *node2,int size)
{
  void *swapmem;
  int nxprsize;

  nxprsize=sizeof(struct nextprev);
  size-=nxprsize;

  node1+=nxprsize;
  node2+=nxprsize;

  swapmem=malloc(size);
  if(malloc==0) logerror(errorlogwindow,"swapnodes(): swapmem==0!");
  else
  {
    memcpy(swapmem,node1,size);
    memcpy(node1,node2,size);
    memcpy(node2,swapmem,size);
  }
  free(swapmem);
}

int getcookie(char *buffer)
{
  FILE *handle;
  unsigned int pointer,numof;

  buffer[0]=0;
  handle=fopen(prefs.cookiefile,"r");

  if(handle==0) printf("Konnte Cookie-Datei '%s' nicht ffnen.\n",prefs.cookiefile);
  else
  {
    numof=0;
    while(feof(handle)==0)
    {
      fgets(buffer,LINELEN,handle);
      numof++;
    } // endwhile

    fseek(handle,0,SEEK_SET);
    pointer=(rand()%numof)+1;

    while(pointer--!=0) fgets(buffer,LINELEN,handle);

    fclose(handle);
    if(strlen(buffer)!=0)
    {
      if(buffer[strlen(buffer)-1]=='\n') buffer[strlen(buffer)-1]=0;         // CR entfernen
      if(buffer[strlen(buffer)-1]=='\r') buffer[strlen(buffer)-1]=0;         // LF entfernen
    }
  }
  return strlen(buffer);
}

void resetautoawaytimer(struct preferences *prefs)
{
  printf("Resetting to %d seconds\n",prefs->autoawaytime);
  fl_set_timer(mainform->AutoAwayTimer,prefs->autoawaytime);
}
